@@ -489,7 +489,10 @@ class PMA_Menu
             $tabs['designer']['id'] = 'designer_tab';
         }
 
-        if (! $db_is_system_schema && $cfgRelation['central_columnswork']) {
+        if (! $db_is_system_schema
+            && isset($cfgRelation['central_columnswork'])
+            && $cfgRelation['central_columnswork']
+        ) {
             $tabs['central_columns']['text'] = __('Central columns');
             $tabs['central_columns']['icon'] = 'centralColumns.png';
             $tabs['central_columns']['link'] = 'db_central_columns.php';
