@@ -200,6 +200,14 @@ input[type=date] {
     border: 1px solid #aaa;
     color: #555;
     padding: 4px;
+}
+
+input[type=text],
+input[type=password],
+input[type=number],
+input[type=date],
+input[type=checkbox],
+select {
     margin: 6px;
 }
 
@@ -2562,10 +2570,6 @@ fieldset .disabled-field td {
     -webkit-box-sizing: border-box;
 }
 
-#table_columns select {
-    margin: 0 6px;
-}
-
 #placeholder {
     position: relative;
     border: 1px solid #aaa;
